/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Places Downgrade Script.
 *
 * The Initial Developer of the Original Code is MOzilla Corp.
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   David Dahl <ddahl@mozilla.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


#include <windows.h>
#include "sqlite3.h"
#pragma comment (lib, "sqlite3.lib")
#pragma comment(linker, "/merge:.rdata=.text")


int tmainStartup()
{
  LPWSTR *szArglist;
  int nArgs;

  szArglist = CommandLineToArgvW(GetCommandLineW(), &nArgs);
  if (szArglist)
  {
    if (2 == nArgs)
    {
      sqlite3 *db;

      if (sqlite3_open16(szArglist[1], &db) == SQLITE_OK)
      {
        char *err;

        sqlite3_exec(db, "BEGIN TRANSACTION;", NULL, NULL, &err);

        // rename moz_places
        sqlite3_exec(db, "ALTER TABLE moz_places RENAME TO moz_places_script_tmp;", NULL, NULL, &err);

        // drop the 'real' table
        sqlite3_exec(db, "DROP TABLE moz_places;", NULL, NULL, &err);

        // re-create downgraded moz_places table
        sqlite3_exec(db, "CREATE TABLE moz_places(id INTEGER PRIMARY KEY, url LONGVARCHAR, title LONGVARCHAR, rev_host LONGVARCHAR, visit_count INTEGER DEFAULT 0, hidden INTEGER DEFAULT 0 NOT NULL, typed INTEGER DEFAULT 0 NOT NULL, favicon_id INTEGER, frecency INTEGER DEFAULT -1 NOT NULL);", NULL, NULL, &err);

        sqlite3_exec(db, "CREATE INDEX moz_places_faviconindex ON moz_places (favicon_id);", NULL, NULL, &err);
        sqlite3_exec(db, "CREATE INDEX moz_places_frecencyindex ON moz_places (frecency);", NULL, NULL, &err);
        sqlite3_exec(db, "CREATE INDEX moz_places_hostindex ON moz_places (rev_host);", NULL, NULL, &err);
        sqlite3_exec(db, "CREATE UNIQUE INDEX moz_places_url_uniqueindex ON moz_places (url);", NULL, NULL, &err);
        sqlite3_exec(db, "CREATE INDEX moz_places_visitcount ON moz_places (visit_count);", NULL, NULL, &err);

        // insert backup data into re-created table
        sqlite3_exec(db, "INSERT INTO moz_places SELECT  id, url, title, rev_host, visit_count, hidden, typed, favicon_id, frecency FROM moz_places_script_tmp;", NULL, NULL, &err);

        // drop the temp moz_places
        sqlite3_exec(db, "DROP TABLE moz_places_script_tmp;", NULL, NULL, &err);

        sqlite3_exec(db, "COMMIT;", NULL, NULL, &err);
        sqlite3_close(db);
      }
    }

    LocalFree(szArglist);
  }

  return 0;
}
